@@ -2,7 +2,7 @@
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
  **********************************************************************
- * Copyright (C) Miroslav Lichvar  2014
+ * Copyright (C) Miroslav Lichvar  2014-2015
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of version 2 of the GNU General Public License as
@@ -32,12 +32,14 @@
 #include "keys.h"
 #include "logging.h"
 #include "manual.h"
+#include "memory.h"
 #include "nameserv.h"
 #include "nameserv_async.h"
 #include "ntp_core.h"
 #include "ntp_io.h"
 #include "ntp_sources.h"
 #include "refclock.h"
+#include "sched.h"
 
 #ifndef FEAT_ASYNCDNS
 
@@ -45,20 +47,43 @@
 
 /* This is a blocking implementation used when asynchronous resolving is not available */
 
-void
-DNS_Name2IPAddressAsync(const char *name, DNS_NameResolveHandler handler, void *anything)
+struct DNS_Async_Instance {
+  const char *name;
+  DNS_NameResolveHandler handler;
+  void *arg;
+};
+
+static void
+resolve_name(void *anything)
 {
+  struct DNS_Async_Instance *inst;
   IPAddr addrs[MAX_ADDRESSES];
   DNS_Status status;
   int i;
 
-  status = DNS_Name2IPAddress(name, addrs, MAX_ADDRESSES);
+  inst = (struct DNS_Async_Instance *)anything;
+  status = DNS_Name2IPAddress(inst->name, addrs, MAX_ADDRESSES);
 
   for (i = 0; status == DNS_Success && i < MAX_ADDRESSES &&
-              addrs[i].family != IPADDR_UNSPEC; i++)
+       addrs[i].family != IPADDR_UNSPEC; i++)
     ;
 
-  (handler)(status, i, addrs, anything);
+  (inst->handler)(status, i, addrs, inst->arg);
+
+  Free(inst);
+}
+
+void
+DNS_Name2IPAddressAsync(const char *name, DNS_NameResolveHandler handler, void *anything)
+{
+  struct DNS_Async_Instance *inst;
+
+  inst = MallocNew(struct DNS_Async_Instance);
+  inst->name = name;
+  inst->handler = handler;
+  inst->arg = anything;
+
+  SCH_AddTimeoutByDelay(0.0, resolve_name, inst);
 }
 
 #endif /* !FEAT_ASYNCDNS */
@@ -75,6 +100,11 @@ CAM_Finalise(void)
 {
 }
 
+void
+CAM_OpenUnixSocket(void)
+{
+}
+
 int
 CAM_AddAccessRestriction(IPAddr *ip_addr, int subnet_bits, int allow, int all)
 {
@@ -169,6 +199,11 @@ NSR_HandleBadSource(IPAddr *address)
 {
 }
 
+void
+NSR_RefreshAddresses(void)
+{
+}
+
 void
 NSR_SetSourceResolvingEndHandler(NSR_SourceResolvingEndHandler handler)
 {
