@@ -42,8 +42,13 @@
 #include <stdio.h>
 #include <signal.h>
 
+#include <mach/mach.h>
+#include <mach/mach_time.h>
+#include <pthread.h>
+
 #include "sys_macosx.h"
 #include "localp.h"
+#include "sched.h"
 #include "logging.h"
 #include "util.h"
 
@@ -69,10 +74,27 @@ static double current_freq;
 
 static double adjustment_requested;
 
-/* Kernel parameters to calculate adjtime error. */
+/* Interval in seconds between adjustments to cancel systematic drift */
 
-static int kern_tickadj;
-static long kern_bigadj;
+#define DRIFT_REMOVAL_INTERVAL (4.0)
+#define DRIFT_REMOVAL_INTERVAL_MIN (0.5)
+
+/* If current_drift_removal_interval / drift_removal_interval exceeds this
+   ratio, then restart the drift removal timer */
+
+#define DRIFT_REMOVAL_RESTART_RATIO (8.0)
+
+static double drift_removal_interval;
+static double current_drift_removal_interval;
+static struct timeval Tdrift;
+
+/* weighting applied to error in calculating drift_removal_interval */
+#define ERROR_WEIGHT (0.5)
+
+/* minimum resolution of current_frequency */
+#define FREQUENCY_RES (1.0e-9)
+
+#define NANOS_PER_MSEC (1000000ULL)
 
 /* ================================================== */
 
@@ -84,10 +106,13 @@ clock_initialise(void)
   offset_register = 0.0;
   adjustment_requested = 0.0;
   current_freq = 0.0;
+  drift_removal_interval = DRIFT_REMOVAL_INTERVAL;
+  current_drift_removal_interval = DRIFT_REMOVAL_INTERVAL;
 
   if (gettimeofday(&T0, NULL) < 0) {
     LOG_FATAL(LOGF_SysMacOSX, "gettimeofday() failed");
   }
+  Tdrift = T0;
 
   newadj.tv_sec = 0;
   newadj.tv_usec = 0;
@@ -112,10 +137,8 @@ start_adjust(void)
 {
   struct timeval newadj, oldadj;
   struct timeval T1;
-  double elapsed, accrued_error;
+  double elapsed, accrued_error, predicted_error, drift_removal_elapsed;
   double adjust_required;
-  struct timeval exact_newadj;
-  long delta, tickdelta;
   double rounding_error;
   double old_adjust_remaining;
 
@@ -127,26 +150,24 @@ start_adjust(void)
   UTI_DiffTimevalsToDouble(&elapsed, &T1, &T0);
   accrued_error = elapsed * current_freq;
 
-  adjust_required = - (accrued_error + offset_register);
+  UTI_DiffTimevalsToDouble(&drift_removal_elapsed, &T1, &Tdrift);
 
-  UTI_DoubleToTimeval(adjust_required, &exact_newadj);
+  /* To allow for the clock being stepped either forward or backwards, clamp
+     the elapsed time to bounds [ 0.0, current_drift_removal_interval ] */
+  drift_removal_elapsed = MIN(MAX(0.0, drift_removal_elapsed), current_drift_removal_interval);
 
-  /* At this point, we need to round the required adjustment the
-     same way the kernel does. */
+  predicted_error = (current_drift_removal_interval - drift_removal_elapsed) / 2.0 * current_freq;
 
-  delta = exact_newadj.tv_sec * 1000000 + exact_newadj.tv_usec;
-  if (delta > kern_bigadj || delta < -kern_bigadj)
-    tickdelta = 10 * kern_tickadj;
-  else
-    tickdelta = kern_tickadj;
-  if (delta % tickdelta)
-	delta = delta / tickdelta * tickdelta;
-  newadj.tv_sec = 0;
-  newadj.tv_usec = (int)delta;
-  UTI_NormaliseTimeval(&newadj);
+  DEBUG_LOG(LOGF_SysMacOSX, "drift_removal_elapsed: %.3f current_drift_removal_interval: %.3f predicted_error: %.3f",
+                            1.0e6 * drift_removal_elapsed,
+                            1.0e6 * current_drift_removal_interval,
+                            1.0e6 * predicted_error);
 
-  /* Add rounding error back onto offset register. */
-  UTI_DiffTimevalsToDouble(&rounding_error, &newadj, &exact_newadj);
+  adjust_required = - (accrued_error + offset_register + predicted_error);
+
+  UTI_DoubleToTimeval(adjust_required, &newadj);
+  UTI_TimevalToDouble(&newadj, &adjustment_requested);
+  rounding_error = adjust_required - adjustment_requested;
 
   if (adjtime(&newadj, &oldadj) < 0) {
     LOG_FATAL(LOGF_SysMacOSX, "adjtime() failed");
@@ -154,10 +175,9 @@ start_adjust(void)
 
   UTI_TimevalToDouble(&oldadj, &old_adjust_remaining);
 
-  offset_register = rounding_error - old_adjust_remaining;
+  offset_register = rounding_error - old_adjust_remaining - predicted_error;
 
   T0 = T1;
-  UTI_TimevalToDouble(&newadj, &adjustment_requested);
 }
 
 /* ================================================== */
@@ -272,33 +292,129 @@ get_offset_correction(struct timeval *raw,
 
 /* ================================================== */
 
+/* Cancel systematic drift */
+
+static int drift_removal_running = 0;
+static SCH_TimeoutID drift_removal_id;
+
+/* ================================================== */
+/* This is the timer callback routine which is called periodically to
+ invoke a time adjustment to take out the machine's drift.
+ Otherwise, times reported through this software (e.g. by running
+ ntpdate from another machine) show the machine being correct (since
+ they correct for drift build-up), but any program on this machine
+ that reads the system time will be given an erroneous value, the
+ degree of error depending on how long it is since
+ get_offset_correction was last called. */
+
+static void
+drift_removal_timeout(SCH_ArbitraryArgument not_used)
+{
+
+  stop_adjust();
+
+  if (gettimeofday(&Tdrift, NULL) < 0) {
+    LOG_FATAL(LOGF_SysMacOSX, "gettimeofday() failed");
+  }
+
+  current_drift_removal_interval = drift_removal_interval;
+
+  start_adjust();
+
+  drift_removal_id = SCH_AddTimeoutByDelay(drift_removal_interval, drift_removal_timeout, NULL);
+}
+
+/* ================================================== */
+
+/* use est_error to calculate the drift_removal_interval */
+
+static void
+set_sync_status(int synchronised, double est_error, double max_error)
+{
+  double interval;
+
+  if (!synchronised) {
+    drift_removal_interval = MAX(drift_removal_interval, DRIFT_REMOVAL_INTERVAL);
+  } else {
+    interval = ERROR_WEIGHT * est_error / (fabs(current_freq) + FREQUENCY_RES);
+    drift_removal_interval = MAX(interval, DRIFT_REMOVAL_INTERVAL_MIN);
+
+    DEBUG_LOG(LOGF_SysMacOSX, "est_error: %.3f current_freq: %.3f est drift_removal_interval: %.3f act drift_removal_interval: %.3f",
+                est_error * 1.0e6, current_freq * 1.0e6, interval, drift_removal_interval);
+  }
+
+  if (current_drift_removal_interval / drift_removal_interval > DRIFT_REMOVAL_RESTART_RATIO) {
+    /* recover from a large est_error by resetting the timer */
+    SCH_ArbitraryArgument unused;
+    SCH_RemoveTimeout(drift_removal_id);
+    unused = NULL;
+    drift_removal_timeout(unused);
+  }
+}
+
+/* ================================================== */
+/*
+  Give chronyd real time priority so that time critical calculations
+  are not pre-empted by the kernel.
+*/
+
+static int
+set_realtime(void)
+{
+  /* https://developer.apple.com/library/ios/technotes/tn2169/_index.html */
+
+  mach_timebase_info_data_t timebase_info;
+  double clock2abs;
+  thread_time_constraint_policy_data_t policy;
+  int kr;
+
+  mach_timebase_info(&timebase_info);
+  clock2abs = ((double)timebase_info.denom / (double)timebase_info.numer) * NANOS_PER_MSEC;
+
+  policy.period = 0;
+  policy.computation = (uint32_t)(5 * clock2abs); /* 5 ms of work */
+  policy.constraint = (uint32_t)(10 * clock2abs);
+  policy.preemptible = 0;
+
+  kr = thread_policy_set(
+          pthread_mach_thread_np(pthread_self()),
+          THREAD_TIME_CONSTRAINT_POLICY,
+          (thread_policy_t)&policy,
+          THREAD_TIME_CONSTRAINT_POLICY_COUNT);
+
+  if (kr != KERN_SUCCESS) {
+    LOG(LOGS_WARN, LOGF_SysMacOSX, "Cannot set real-time priority: %d", kr);
+    return -1;
+  }
+  return 0;
+}
+
+/* ================================================== */
+
+void
+SYS_MacOSX_SetScheduler(int SchedPriority)
+{
+  if (SchedPriority) {
+    set_realtime();
+  }
+}
+
+/* ================================================== */
+
 void
 SYS_MacOSX_Initialise(void)
 {
-  int result;
-  size_t len;
-  struct clockinfo clockinfo;
-  int mib[2];
-
-  mib[0] = CTL_KERN;
-  mib[1] = KERN_CLOCKRATE;
-
-  len = sizeof(clockinfo);
-  result = sysctl(mib, 2, &clockinfo, &len, NULL, 0);
-
-  if(result < 0) {
-    LOG_FATAL(LOGF_SysMacOSX, "Cannot read clockinfo");
-  }
-  kern_tickadj = clockinfo.tickadj;
-  kern_bigadj = clockinfo.tick;
-
   clock_initialise();
 
   lcl_RegisterSystemDrivers(read_frequency, set_frequency,
                             accrue_offset, apply_step_offset,
                             get_offset_correction,
                             NULL /* set_leap */,
-                            NULL /* set_sync_status */);
+                            set_sync_status);
+
+
+  drift_removal_id = SCH_AddTimeoutByDelay(drift_removal_interval, drift_removal_timeout, NULL);
+  drift_removal_running = 1;
 }
 
 /* ================================================== */
@@ -306,6 +422,10 @@ SYS_MacOSX_Initialise(void)
 void
 SYS_MacOSX_Finalise(void)
 {
+  if (drift_removal_running) {
+    SCH_RemoveTimeout(drift_removal_id);
+  }
+
   clock_finalise();
 }
 
